

DJANGO_DOMAIN               = 'django.python.unimatrixone.io'
DJANGO_PROJECT_NAME         = f'{DJANGO_DOMAIN}/project'
DJANGO_SETTINGS_MODULE      = f'{DJANGO_DOMAIN}/settings'
PLATFORM_DOMAIN             = "platform.unimatrixone.io"
PLATFORM_LANGUAGE_NAME      = f"{PLATFORM_DOMAIN}/language-name"
PLATFORM_LANGUAGE_VERSION   = f"{PLATFORM_DOMAIN}/language-version"
PLATFORM_FRAMEWORK_NAME     = f"{PLATFORM_DOMAIN}/framework-name"
PLATFORM_FRAMEWORK_VERSION  = f"{PLATFORM_DOMAIN}/framework-version"
PLATFORM_OS_NAME            = f"{PLATFORM_DOMAIN}/os-name"
PLATFORM_OS_RELEASE         = f"{PLATFORM_DOMAIN}/os-release"
